@@ -12,12 +12,12 @@
  
  
  Peter Beerli 1997, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: broyden.h 175 2005-12-02 14:51:23Z beerli $
+$Id: broyden.h 514 2006-11-09 20:14:08Z beerli $
  
 -------------------------------------------------------*/
 
