@@ -17,12 +17,12 @@
   
 Copyright 2002 Peter Beerli and Joseph Felsenstein
  
-$Id: definitions.h 762 2007-07-12 22:25:28Z beerli $
+$Id: definitions.h 1090 2007-12-25 20:02:46Z beerli $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "2.3"
+#define MIGRATEVERSION "2.3.3"
 #else
 #ifdef SGI
 #undef MIGRATEVERSION
@@ -37,12 +37,18 @@ $Id: definitions.h 762 2007-07-12 22:25:28Z beerli $
 /*------------------------------------------------
   Adjusting the source to the system configuration
 */
+#ifdef use_floats
+// use this when using floats 
+#define USE_MYREAL_FLOAT 0
+#define MYREAL float
+#define MYFLOAT float
+#define MYREAL_MAX 3.40282347e+38F
+#else
 #define USE_MYREAL_DOUBLE 1
 #define MYREAL double
-// use this when using floats 
-//#define USE_MYREAL_FLOAT 0
-//#define MYREAL float
-
+#define MYFLOAT float
+#define MYREAL_MAX 1.7976931348623157e+308
+#endif
 #ifdef HAVE_CONFIG_H
 #include "conf.h"
 #endif
@@ -93,6 +99,7 @@ $Id: definitions.h 762 2007-07-12 22:25:28Z beerli $
     #undef HAVE_LGAMMA
     #define HAVE_STRFTIME
     #undef HAVE_STRSEP
+#include "windows_timveval.h"
 #endif
 
 /* we have found no strftime() and replace the time
@@ -197,17 +204,20 @@ void vax_tweak_fmt (char *);
 #ifdef WIN32
     #define MYINLINE __inline
     #undef DBL_EPSILON
-    #undef DBL_MAX
+//    #undef MYREAL_MAX
+#define __restrict /*__restrict*/
     #include <float.h>
     #define MYISNAN _isnan
+    #define MYFINITE _finite
     #define MYSNPRINTF _snprintf
 #else  
-	#define MYISNAN  isnan
+    #define MYISNAN  isnan
+    #define MYFINITE finite
     #define MYSNPRINTF snprintf
     #ifdef __MWERKS__
 		#define MYINLINE /*inline*/
 		#undef DBL_EPSILON
-		#undef DBL_MAX
+		#undef MYREAL_MAX
 		#include <float.h>
 	#else
 		#define MYINLINE inline
@@ -233,9 +243,16 @@ void vax_tweak_fmt (char *);
 
 //#include "sighandler.h"
 #define FClose(file) if (file) fclose(file) ; file=NULL
+#ifdef WIN32
+#ifndef WINDOWS
+#define WINDOWS
+#endif
+#include <windows.h>
+#include "windows_timeval.h"
+#else
 typedef int boolean;
 //typedef unsigned char boolean;
-
+#endif
 #ifndef TRUE
 #define TRUE    (boolean) 1
 #endif
@@ -303,7 +320,7 @@ typedef int boolean;
    observed alleles, much bigger does not harm */
 #define MAXALLELES            93L
 #define SMALLBUFSIZE    255L
-#define MAXBUFSIZE        100000L
+#define MAXBUFSIZE        1000000L
 #define LINESIZE            10000L /* setting this smaller can break */
 #define LONGLINESIZE       10000L /* setting this smaller can break */
 #define SUPERLINESIZE    100000L /*used to read many many loci-sites and indiv numbers*/ 
@@ -336,6 +353,7 @@ typedef int boolean;
 so that the given number of
 long chains is used */
 #define CHAINVARIANCEDELTA 0.01 /*difference of variances of parts of a chain [see burnin_autostop]*/
+#define ESSMINIMUM 100  /*effective sample size minimum see burnin_autostop*/
 #define GELMAN_MYSTIC_VALUE 1.2
 #define PLUSCHAIN 10
 #ifndef MAXLONG
@@ -359,6 +377,8 @@ long chains is used */
 #define LOG2 0.693147180559945309417232121458 /*N[Log[2] ,30] */
 #define LOG2PIHALF -0.918938533204672741780329736406 /*N[Log[1/Sqrt[2 Pi]] ,30] */
 #define TWOPI 6.28318530717958647692528676656 /*N[Log[2 Pi] ,30] */
+#define PI 3.14159265358979323846264338328 /*N[Pi,30]*/
+#define INV2PI 0.159154943091895335768883763373 /*N[1/(2 pi)]*/
 #define ROOTLENGTH         10000
 #define FIRSTCHAIN            -1
 #define FIRSTSTEP             -1
@@ -407,7 +427,7 @@ long chains is used */
 #define SMALLEST_GAMMA      1e-3
 #define BIGGEST_GAMMA        1e9
 #define FRACTION_ALONG      0.66
-#define TIMELIST_GUESS       100
+#define TIMELIST_GUESS       500
 #define SAMPLETREE_GUESS       1
 #define START_ALPHA          10.0
 /* random material */
