@@ -8,7 +8,7 @@ Peter Beerli 2000
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: heating.h 514 2006-11-09 20:14:08Z beerli $
+$Id: heating.h 1049 2007-11-29 22:47:41Z beerli $
  
 */
 
@@ -54,7 +54,7 @@ extern void fill_tpool (tpool_t tpool, world_fmt ** universe,
 extern void wait_tpool (tpool_t tpoolp, int usize);
 extern int tpool_destroy (tpool_t tpool, int finish);
 extern int tpool_synchronize (tpool_t tpool, int finish);
-extern int tpool_add_work (tpool_t tpool, void *routine, void *arg);
+extern int tpool_add_work (tpool_t tpool, void *routine, void *arg, long z);
 #else
 
 
