@@ -26,16 +26,17 @@
 *   works and am not responsible for any damage you may cause or have.
 *
 */
-/* $Id: main.c 758 2007-07-03 14:12:40Z beerli $*/
+/* $Id: main.c 1102 2008-01-01 23:45:34Z beerli $*/
 
 #include "migration.h"
 #include "sighandler.h"
 #include "heating.h"
-#include "bayes.h"
+#include "histogram.h"
 #include "world.h"
 #include "data.h"
 #include "options.h"
 #include "mcmc.h"
+#include "bayes.h"
 #include "broyden.h"
 #include "combroyden.h"
 #include "menu.h"
@@ -73,19 +74,30 @@ extern void set_haru_handler(void);
 #endif
 
 /* Definitions for MCMCMC -- heated chains
-* definitions of the first four heated chains, they are ordered from cold to hot */
+ * definitions of the first four heated chains, they are ordered from cold to hot */
 #define EARTH  universe[0]	/*!< cold chain has always a temperature=1            */
 #define VENUS  universe[1]	/*!< second coldest chain                             */
 #define MERKUR universe[2]	/*!< thrid coldest chain                              */
 #define SUN    universe[3]	/*!< fourth coldest chain                             */
 
-/* GLOBAL VARIABLES ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
-int myID;			/*!< myID=0 for single-cpu program and master in MPI program, worker nodes myID > 0 */
-int myRepID;			/*!< myID=0 for single-cpu program and master in MPI program, worker nodes myID > 0 */
-int color;			/*!< either 1 or MPI_UNDEFINED, in anticipation of more complicated MPI schemes    */
-int numcpu;			/*!< used if MPI is running otherwise == 1                                         */
-int locidone;			/*!< used in MPI workers                                                           */
+/* GLOBAL VARIABLES +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
+int myID;	  /*!< myID=0 for single-cpu program and master in MPI program, worker nodes myID > 0 */
+int myRepID;	  /*!< myID=0 for single-cpu program and master in MPI program, worker nodes myID > 0 */
+int color;	  /*!< either 1 or MPI_UNDEFINED, in anticipation of more complicated MPI schemes     */
+int numcpu;	  /*!< used if MPI is running otherwise == 1                                          */
+int locidone;	  /*!< used in MPI workers                                                            */
+long unique_id_global;
+#ifdef MPI
+#ifdef  USE_MYREAL_FLOAT
+const MPI_Datatype mpisizeof = MPI_FLOAT ;
+#else
+const MPI_Datatype mpisizeof = MPI_DOUBLE ;
+#endif
+#endif
 
+#ifdef CAUTIOUS
+boolean cautious; /*!< forces a check whether a file exists and will ask before overwriting           */
+#endif
 #ifdef MEMDEBUG
 #include <sys/time.h>
 FILE *memfile;
@@ -129,13 +141,12 @@ pdf_page page;
 pdf_contents canvas;
 int page_counter;
 char pdf_pagetitle[LINESIZE+1];
+char pdf_time[LINESIZE+1];
 float page_height;
 float page_width;
 #endif
 
 
-
-
 int
 setup_locus (long locus, world_fmt * world, option_fmt * options,
              data_fmt * data);
@@ -332,9 +343,13 @@ main (int argc, char **argv)
     generator = (char *) mycalloc (1,sizeof(char) * 80);
     universe = (world_fmt **) mycalloc (HEATED_CHAIN_NUM, sizeof (world_fmt *));
     
+    // set flag to decide whether we use caution when writing files
+#ifdef CAUTIOUS
+    cautious = FALSE;
+#endif
     // try to catch and beautify some error messages
     signalhandling (ON);
-    
+    unique_id_global=0;
     // create main data structures
     create_data (&data);
     create_world (&(EARTH), 1L);
@@ -393,8 +408,16 @@ main (int argc, char **argv)
         get_menu (options);
         if (options->bayes_infer)
 	  {
-            options->schains = 0;
-	    options->lchains = 1;
+	    if(options->schains>0)
+	      {
+		//		fprintf(stdout,"NOTICE: with Bayesian inference no short chains are allowed, setting them to ZERO\n");
+		options->schains = 0;
+	      }
+	    if(options->lchains>1 || options->lchains < 1)
+	      {
+		fprintf(stdout,"NOTICE: with Bayesian inference, most efficient runs are with 1 long chain,\nsetting the long chain to 1\n");
+		options->lchains = 1;
+	      };
 	  }
 #ifdef MPI        
 	MYMPIBARRIER (comm_world);
@@ -440,8 +463,15 @@ main (int argc, char **argv)
     }
     else
     {
+      if(EARTH->options->bayes_infer)
+	{
+	  //	  analyze_oldbayesdata(EARTH, options, data, &outfilepos);
+	}
+      else
+	{
         //reanalyze sumfile
         Gmax = analyze_olddata (EARTH, options, data, &outfilepos);
+	}
     }
     unset_penalizer_function (TRUE);
 #ifdef MPI
@@ -453,7 +483,34 @@ main (int argc, char **argv)
 
     if (options->bayes_infer)
     {
-        get_bayeshist (EARTH, options);
+      if(EARTH->options->has_bayesmdimfile)
+	{
+	  get_bayeshist (EARTH, options); // this is transferring some material but
+	  // the parameter file is not filled in anymore [-> small data block]
+	  // save parameters in file and reload
+	  if(myID==MASTER)
+	    {
+	      FClose(EARTH->bayesmdimfile);
+	      openfile(&EARTH->bayesmdimfile, options->bayesmdimfilename,"r+",NULL);
+	      read_bayes_fromfile(EARTH->bayesmdimfile,EARTH, options);
+	      for(locus=0;locus<EARTH->loci;locus++)
+		{
+		  if(EARTH->data->skiploci[locus])
+		    continue;
+		  if(EARTH->bayes->histogram[locus].results == NULL)
+		    {
+		      EARTH->data->skiploci[locus] = TRUE;
+		      continue;
+		    }
+		  calc_hpd_credibility(EARTH->bayes, locus, EARTH->numpop2 + EARTH->bayes->mu);
+		}
+	    }
+	}
+      else
+	{
+	  // keep everything in RAM
+	  get_bayeshist (EARTH, options);
+	}
     }
     else
     {
@@ -485,11 +542,12 @@ main (int argc, char **argv)
 #ifdef MPI
         get_mighistdata (EARTH, options);
 #endif
-	if(options->skyline)
+	if(options->skyline && options->bayes_infer)
 	  print_expected_values(EARTH);       
 	if(options->mighist || options->skyline)
 	  {
 	    print_event_values(EARTH);
+	    pdf_histogram_legend();
 	  }        
 
 
@@ -504,7 +562,13 @@ main (int argc, char **argv)
         unset_penalizer_function (TRUE);	//now we calculate profile and shut down
                                             // the penalizing of far jumps in the maximizer function
     }
-    
+
+#ifdef NEXUSTREE
+    if (EARTH->options->treeprint == LASTCHAIN)
+      {
+	FPRINTF(EARTH->treefile,"\nend;\n");
+      }
+#endif
 #ifdef SLOWNET
     //    if(myID == MASTER)
     //{
@@ -519,10 +583,18 @@ main (int argc, char **argv)
             // the workers will stop working on locus-likelihoods and advance to profiles
             mpi_send_stop (EARTH);
         }
-    
+#ifndef INTEGRATEDLIKE    
         if (!options->bayes_infer)
-            profile_tables (options, EARTH, &Gmax);
-	//}
+	  {
+#endif
+	    if (EARTH->options->lratio->counter > 0)
+	      print_lratio_test (EARTH, &Gmax);
+	    if (options->aic)
+	      akaike_information (EARTH, &Gmax);
+	    profile_tables (options, EARTH, &Gmax);
+#ifndef INTEGRATEDLIKE    
+	  }
+#endif
 #else  /* SLOWNET*/
 
         if (myID == MASTER)
@@ -534,6 +606,12 @@ main (int argc, char **argv)
 			       && options->replicatenum >
 			       0) ? options->replicatenum : 1;
 	       print_theta0(stdout, EARTH, maxreplicate);
+
+	       if (EARTH->options->lratio->counter > 0)
+		 print_lratio_test (EARTH, &Gmax);
+	       if (options->aic)
+		 akaike_information (EARTH, &Gmax);
+	       
 	       profile_tables (options, EARTH, &Gmax);
 	     }
 #else
@@ -549,32 +627,46 @@ main (int argc, char **argv)
     if (myID == MASTER)
         {
 #endif
+	  // print marginal likelihoods
 	  if(EARTH->options->bayes_infer)
 	    {
-	      fprintf(EARTH->outfile,"MCMC run characteristics: Autocorrelation and Effective sample size\n");
+	      print_bayesfactor(EARTH->outfile, universe,options);
+	    }
+
+	  // printing of MCMC run characteristica
+
+	  if(EARTH->options->bayes_infer)
+	    {
+	      fprintf(EARTH->outfile,"MCMC run characteristics\n");
+	      fprintf(EARTH->outfile,"========================\n\n");
+	      bayes_print_accept(EARTH->outfile,EARTH);
+#ifdef PRETTY
+	      pdf_bayes_print_accept(EARTH);
+#endif
+	      fprintf(EARTH->outfile,"Autocorrelation and Effective sample size\n");
 	      fprintf(EARTH->outfile,"-------------------------------------------------------------------\n\n");
-	      print_param_eff_size(EARTH->outfile,EARTH,EARTH->numpop2,2,EARTH->auto_archive, EARTH->ess_archive);
+	      print_param_eff_size(EARTH->outfile,EARTH,EARTH->numpop2 + EARTH->bayes->mu * EARTH->loci + 1,2,EARTH->auto_archive, EARTH->ess_archive);
 #ifdef PRETTY
 	      pdf_bayes_print_ess(EARTH);
 #endif
 	    }
-#ifdef NEWVERSION
-	  print_bayesfactor(EARTH->outfile, universe,options);
-	  if(EARTH->options->burnin_autostop == 'a')
+
+	  if(strchr("ae",EARTH->options->burnin_autostop))
 	  {
 	    print_burnin_stop(EARTH->outfile, universe, options);
 	  }
-#endif /*NEWVERSION*/
+
 	  if(EARTH->options->progress)
 	    {
 	      if(EARTH->options->bayes_infer)
 		{
 		  fprintf(stdout,"\nMCMC run characteristics: Autocorrelation and Effective sample size\n");
 		  fprintf(stdout,"-------------------------------------------------------------------\n\n");
-		  print_param_eff_size(stdout,EARTH,EARTH->numpop2,2,EARTH->auto_archive, EARTH->ess_archive);
-#ifdef NEWVERSION
+		  print_param_eff_size(stdout,EARTH,EARTH->numpop2+ EARTH->bayes->mu * EARTH->loci + 1,2,
+				       EARTH->auto_archive, EARTH->ess_archive);
+
 		  print_bayesfactor(stdout, universe,options);
-#endif /*NEWVERSION*/
+
 		}
 	    }
 	  if(EARTH->options->treeinmemory)
@@ -612,10 +704,12 @@ main (int argc, char **argv)
 	destroy_data(data);
 	destroy_options(options);
 	// for debugging with MallocDebug on macosx
-	//	while(TRUE)
-	// {
-	//  sleep(1);
-	//} //end for debugging
+	#ifdef MACMALLOCDEBUG
+	while(TRUE)
+	 {
+	  sleep(1);
+	} //end for debugging
+	#endif
         return 0;
 }				/* main end */
 
@@ -682,6 +776,12 @@ combine_loci_mpi (char type, option_fmt * options, world_fmt * world,
         world->repkind = set_repkind (options);
         if (myID == MASTER)
         {
+#ifdef INTEGRATEDLIKE
+	      mpi_gmax_master (world, Gmax);
+	      mpi_startparam_master (world);
+	      decide_plot (world->options, world->options->lchains,
+			   world->options->lchains, 'l');
+#else
 	  if(!options->bayes_infer)
 	    {
 	      mpi_gmax_master (world, Gmax);
@@ -689,11 +789,12 @@ combine_loci_mpi (char type, option_fmt * options, world_fmt * world,
 	      decide_plot (world->options, world->options->lchains,
 			   world->options->lchains, 'l');
 	    }
+#endif
 #ifdef LONGSUM
             change_longsum_times (world);	//multilocus
 #endif	   /*LONGSUM*/
             // broadcast Gmax to all workers, the worker pendant is in mpi_gmax_worker
-
+#ifndef INTEGRATEDLIKE
 	    if (!options->bayes_infer)
 	      {
 		MYMPIBCAST (Gmax, 1, MPI_LONG, MASTER, comm_world);
@@ -702,9 +803,17 @@ combine_loci_mpi (char type, option_fmt * options, world_fmt * world,
 					  0 /* chain */ ,
 					  type, kind, world->repkind);
 	      }
+#else
+		MYMPIBCAST (Gmax, 1, MPI_LONG, MASTER, comm_world);
+		(void) estimateParameter (options->replicatenum, *Gmax, world,
+					  options, world->cov[world->loci],
+					  0 /* chain */ ,
+					  type, kind, world->repkind);
+#endif
 	}
         else
         {
+#ifndef INTEGRATEDLIKE
             //            printf("%i> before gmax worker\n", myID);
 	  if(!options->bayes_infer)
 	    {
@@ -713,6 +822,11 @@ combine_loci_mpi (char type, option_fmt * options, world_fmt * world,
 	    }
             //            printf("%i> start worker postlike calculations\n", myID);
             mpi_maximize_worker (world, MULTILOCUS, options->replicatenum);	// receive broadcast for Gmax
+#else
+	    mpi_gmax_worker (world);
+	    mpi_startparam_worker (world);
+            mpi_maximize_worker (world, MULTILOCUS, options->replicatenum);	// receive broadcast for Gmax
+#endif
         }
     }
     //   else
@@ -737,20 +851,21 @@ void
 run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
              int usize, long *outfilepos, long *Gmax)
 {
-    long i;
-    long maxreplicate;
-    long treefilepos;
+  MYREAL var;
+  long i;
+  long maxreplicate;
+  long treefilepos;
 #ifdef PRETTY
-    float left_margin;
+  float left_margin;
 #endif /*PRETTY*/
 #ifdef MPI
-    //#ifdef MPIREPLICANT
-    //    MPI_Status status;		// contains status used in MPI_Recv
-    MPI_Request *irequests;	// contains requests generated in MPI_Isend
-    MPI_Status *istatus;		// conatins stata from MPI_Wait_any
-    long minnodes;
-    //    long sent;			// counter for the array irequests of the MPI_Isend requests
-                        //#endif
+  //#ifdef MPIREPLICANT
+  //    MPI_Status status;		// contains status used in MPI_Recv
+  MPI_Request *irequests;	// contains requests generated in MPI_Isend
+  MPI_Status *istatus;		// conatins stata from MPI_Wait_any
+  long minnodes;
+  //    long sent;			// counter for the array irequests of the MPI_Isend requests
+  //#endif
     long *twolongs;		// message sent to workers contains locus and replicate number
                         //int          *ranks;
                         //MPI_Group worldgroup, locigroup;
@@ -789,6 +904,16 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
     EARTH->cold = TRUE; // this is the cold chain when we use heating, the hotter chains have FALSE
     init_world (EARTH, data, options);
     set_meanmu(EARTH,options);
+    if(EARTH->cold)
+      single_chain_var (EARTH, 0, &var, NULL, NULL);
+
+#ifdef NEXUSTREE
+      if (EARTH->options->treeprint == LASTCHAIN)
+	{
+	  FPRINTF(EARTH->treefile,"#NEXUS\n\nbegin trees;");
+	}
+#endif
+
 #ifdef PTHREADS
     tpool_init (&heating_pool, usize, usize, 0);
 #endif
@@ -968,14 +1093,19 @@ void
 get_bayeshist (world_fmt * world, option_fmt * options)
 {
 #ifdef MPI
+  //long i;
     long maxreplicate = (options->replicate
                          && options->replicatenum >
                          0) ? options->replicatenum : 1;
     
     if (myID == MASTER)
     {
+      //for(i=0;i<world->loci;i++)
+      //	  printf("%i> before moving stuff  %f %f\n",myID, world->hm[i],world->am[i]);
         mpi_results_master (MIGMPI_BAYESHIST, world, maxreplicate,
                             unpack_bayes_buffer);
+	//for(i=0;i<world->loci;i++)
+	//  printf("%i> after moving stuff  %f %f\n",myID, world->hm[i],world->am[i]);
     }
 #endif
 }
@@ -1136,8 +1266,11 @@ print_heating_progress2 (FILE * file, worldoption_fmt * options,
   char *plog;
   long plogsize = 0L;
   plog = (char *) mycalloc(LONGLINESIZE,sizeof(char));
-  
+#ifdef MPI
+  plogsize = sprintf(plog, "[%3i] Swapping between %li temperatures.\n", myID, options->heated_chains);
+#else
   plogsize = sprintf(plog, "           Swapping between %li temperatures.\n", options->heated_chains);
+#endif
   if (options->heated_chains > 4)
     plogsize += sprintf (plog + plogsize, "               Only the 4 coldest chains are shown\n");
   plogsize += sprintf (plog + plogsize,"           Temperature | Accepted |  Swaps between temperatures\n");
@@ -1194,8 +1327,8 @@ analyze_olddata (world_fmt * world,
         
 #ifdef MPI
         
-        MYMPIBCAST (data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
-        MYMPIBCAST (data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+        MYMPIBCAST (data->geo, world->numpop2, mpisizeof, MASTER, comm_world);
+        MYMPIBCAST (data->lgeo, world->numpop2, mpisizeof, MASTER, comm_world);
     }
     else
     {
@@ -1212,8 +1345,8 @@ analyze_olddata (world_fmt * world,
         data->geo = (MYREAL *) mycalloc (1, sizeof (MYREAL) * world->numpop2);
         data->lgeo = (MYREAL *) mycalloc (1, sizeof (MYREAL) * world->numpop2);
         
-        MYMPIBCAST (data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
-        MYMPIBCAST (data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+        MYMPIBCAST (data->geo, world->numpop2, mpisizeof, MASTER, comm_world);
+        MYMPIBCAST (data->lgeo, world->numpop2, mpisizeof, MASTER, comm_world);
         options->muloci = data->loci = world->loci;
         init_world (world, data, options);
 #endif
@@ -1414,15 +1547,23 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
     print_menu_locus (world->options->logfile, world, locus);
   world->start = TRUE;
   buildtree (world, options, data, locus);
-  if (world->replicate == 0)
+  /*  if (world->replicate == 0)
     {
       if (strchr (SNPTYPES, options->datatype))
         {
+	  // unlinked snps -- does this work at all
 	  if (options->datatype == 'u')
-	    world->data->seq->endsite *= (data->seq->addon + 1);
-	  world->data->seq->endsite += data->seq->addon;
-        }
-    }
+	    {
+	      world->data->seq->endsite *= (data->seq->addon + 1);
+	    }
+	  if (options->datatype == 'n')
+	    {
+	      world->data->seq->endsite += data->seq->addon;
+	    }
+	}
+	}
+  */
+
   //    if (!options->replicate
   //    || (options->replicate && options->replicatenum == 0)
   //    || (options->replicate && world->replicate == options->replicatenum))
@@ -1432,9 +1573,12 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
   //}
   create_treetimelist (world, &world->treetimes, locus);
   fix_times (world, options);
+#ifdef DEBUG
+  //  printf(" fixed times \n");
+#endif
   first_smooth (world, locus);
-  world->likelihood[0] = treelikelihood (world);
-  world->allikemax = -DBL_MAX;	/* for best tree option */
+  world->likelihood[0] = -MYREAL_MAX; //treelikelihood (world);
+  world->allikemax = -MYREAL_MAX;	/* for best tree option */
 #ifdef UEP
   if (options->uep)
     {
@@ -1457,7 +1601,7 @@ condense_time (world_fmt * world, long *step, long *j, MYREAL * accepted,
 #ifdef INTEGRATEDLIKE
   long i;
   long z;
-  long nn = world->numpop2 + world->bayes->mu;
+  long nn = world->numpop2 + world->bayes->mu * world->loci;
   MYREAL x;
   MYREAL delta;
 #endif
@@ -1466,6 +1610,9 @@ condense_time (world_fmt * world, long *step, long *j, MYREAL * accepted,
     {
       if (world->in_last_chain)
 	{
+	  if (*step == 0)
+	    return;
+
 	  // syncs treeupdate and paramupdate
 	  if(world->bayesaccept == -1)
 	    {
@@ -1476,6 +1623,7 @@ condense_time (world_fmt * world, long *step, long *j, MYREAL * accepted,
 	  *accepted += *j;
 #ifdef INTEGRATEDLIKE
 	  n += 1;
+	  //	  fprintf(stdout,"\naverage MP-values: %li ", n);
 	  for(i=0; i < nn; i++)
 	    {
 	      if(world->bayes->map[i][1] == INVALID)
@@ -1487,7 +1635,9 @@ condense_time (world_fmt * world, long *step, long *j, MYREAL * accepted,
 	      x = world->param0[z];
 	      delta = x - world->atl[rep][world->locus].param0[z];
 	      world->atl[rep][world->locus].param0[z] += delta/n;
+	      //  fprintf(stdout,"%f ",world->atl[rep][world->locus].param0[z]);
 	    }
+	  //	  fprintf(stdout,"\n");
 #else 
 	  return;
 #endif
@@ -1592,7 +1742,11 @@ heating_prepare (world_fmt ** universe, int usize,
     long chain;
     EARTH->heat = options->heat[0];
     for (chain = 1; chain < usize; ++chain)
+      {
+	buildtree (universe[chain], options, data, EARTH->locus);//DEBUG
+
         klone (EARTH, universe[chain], options, data, options->heat[chain]);
+      }
 }
 
 /// prepare for heating scheme: Step I [this function failed in the parallel run, unused for now, replaced by above]
@@ -1696,7 +1850,8 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
       EARTH->repkind = MULTIPLERUN;
       if (options->bayes_infer)
         {
-	  calculate_credibility_interval (EARTH, locus);
+	  if(!options->has_bayesmdimfile)
+	    calculate_credibility_interval (EARTH, locus);
 	  //	  return;
         }
 #ifdef LONGSUM
@@ -1722,8 +1877,12 @@ run_locus (world_fmt ** universe, int usize, option_fmt * options,
     {
       if (options->bayes_infer)
         {
-	  calculate_credibility_interval (EARTH, locus);
-        }
+	  if(!options->has_bayesmdimfile)
+	    {
+	      adjust_bayes_bins(EARTH,locus);
+	      calculate_credibility_interval (EARTH, locus);
+	    }
+	}
     }
   if (options->heating)
     {
@@ -1856,11 +2015,11 @@ run_steps (world_fmt ** universe,
            option_fmt * options,
            tpool_t * localheating_pool, long increment, long steps, long rep)
 {
-    long step;
+    long step=0;
     long oldsteps = steps;
     long ii;
     MYREAL var=0.0;
-    if(EARTH->options->bayes_infer)
+    if(EARTH->cold && EARTH->options->bayes_infer)
       single_chain_var (NULL, step, &var, NULL, NULL);
     for (step = 0; step < steps + 1; step++)
     {
@@ -1871,12 +2030,17 @@ run_steps (world_fmt ** universe,
         store_uep (EARTH);
 #endif
 	//	printf("step=%li inc=%li locus=%li\n",step,increment,EARTH->locus);
+
         condense_time (EARTH, &step, &EARTH->accept,
                        &EARTH->accept_freq, &EARTH->G, &steps, oldsteps, rep);
-	calculate_BF(universe,options);
 	if(EARTH->options->bayes_infer)
-	  single_chain_var (EARTH, step, &var, EARTH->autocorrelation, EARTH->effective_sample);
-
+	  {
+	    calculate_BF(universe,options);
+	    if(EARTH->cold)
+	      {
+		single_chain_var (EARTH, step, &var, EARTH->autocorrelation, EARTH->effective_sample);
+	      }
+	  }
         if (step > 0)
         {
             advance_clone_like (EARTH, EARTH->accept, &EARTH->G);
@@ -1923,6 +2087,7 @@ run_chains (world_fmt ** universe,
       if (options->heating)
 	MERKUR->swapped = VENUS->swapped = EARTH->swapped = 0;
       rep = replicate_number (options, chain, type, replicate);
+      EARTH->rep = rep;
       precalc_world (EARTH);
       polish_world (EARTH);
       burnin_chain (EARTH);
@@ -2100,10 +2265,12 @@ run_replicate (long locus,
 	  }
         }
     }				//end runs
+    //#ifndef MPI
     if(EARTH->cold && EARTH->options->bayes_infer)
       {
 	  collect_eff_values(EARTH);
       }
+    //#endif
 }
 
 
@@ -2115,17 +2282,32 @@ void print_burnin_stop(FILE *file, world_fmt **universe, option_fmt * options)
 		       && options->replicatenum >
 		       0) ? options->replicatenum : 1;
   fprintf(file,"\n\n\nStop of burn-in phase due to convergence\n");
-  fprintf(file,"----------------------------------------\n\n");
+  fprintf(file,"----------------------------------------\n");
+  switch(options->burnin_autostop)
+    {
+    case 'a':
+      fprintf(file,"[Stopping criteria was: Variance ratio between the last two groups of 1000 steps < %f]\n\n",world->varheat); break;
+    case 'e':
+      fprintf(file,"[Stopping criteria was: All effective MCMC sample sizes > %f]\n\n",world->essminimum); 
+      break;
+    case ' ':
+      fprintf(file,"\n\n");
+      break;
+    }
+
   fprintf(file,"Locus  Replicate  Steps  Variance ratio (new/old variance)\n");
   fprintf(file,"-----  ---------  ------ ---------------------------------\n");
  for(z=0; z < world->loci * maxreplicate; z++)
     {
-      fprintf(file,"%5li  %5li  %10li   %10.4f (%f/%f)\n",world->burnin_stops[z].locus,
-	      world->burnin_stops[z].replicate,
-	      world->burnin_stops[z].stopstep,
-	      world->burnin_stops[z].variance/world->burnin_stops[z].oldvariance,
-	      world->burnin_stops[z].variance,
-	      world->burnin_stops[z].oldvariance);
+      if(world->burnin_stops[z].oldvariance > 0.0)
+	{
+	  fprintf(file,"%5li  %5li  %10li   %10.4f (%f/%f)\n",1 + world->burnin_stops[z].locus,
+		  world->burnin_stops[z].replicate,
+		  world->burnin_stops[z].stopstep,
+		  world->burnin_stops[z].variance/world->burnin_stops[z].oldvariance,
+		  world->burnin_stops[z].variance,
+		  world->burnin_stops[z].oldvariance);
+	}
       //			    world->burnin_stops[z].worker = myID;
     }
   fprintf(file,"\n\n");
@@ -2139,62 +2321,85 @@ void print_burnin_stop(FILE *file, world_fmt **universe, option_fmt * options)
 void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 {
   long t;
+  long hc = EARTH->options->heated_chains;
+  long locus;
   long maxreplicate = (options->replicate
 		       && options->replicatenum >
 		       0) ? options->replicatenum : 1;
-  MYREAL sum = 0.;
+  //  MYREAL sum = 0.;
   MYREAL heat0 = 1.;
   MYREAL heat1 = 1.;
-
+  MYREAL bfsum = 0.;
+  MYREAL hsum = 0.;
+  MYREAL asum = 0.;
+  MYREAL lsum;
+  fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood)\n");
+  fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
+  
   if(options->heating)
     {
-      for(t=1; t < EARTH->options->heated_chains; t++)
+      for(locus=0;locus<EARTH->loci;locus++)
 	{
+	  if(EARTH->data->skiploci[locus])
+	    continue;
+
+	  lsum = 0.;
+	  for(t=1; t < hc; t++)
+	    {
 #ifdef MPI
-	  heat0 = 1./options->heat[t-1] ;
-	  heat1 = 1./options->heat[t];
+	      heat0 = 1./options->heat[t-1] ;
+	      heat1 = 1./options->heat[t];
 #else
-	  if(options->adaptiveheat)
-	    {
-	      heat0 = universe[t-1]->averageheat ;
-	      heat1 = universe[t]->averageheat;
-	    }
-	  else
-	    {
-	      heat0 = universe[t-1]->heat ;
-	      heat1 = universe[t]->heat;
-	    }
+	      if(options->adaptiveheat)
+		{
+		  heat0 = universe[t-1]->averageheat ;
+		  heat1 = universe[t]->averageheat;
+		}
+	      else
+		{
+		  heat0 = universe[t-1]->heat ;
+		  heat1 = universe[t]->heat;
+		}
 #endif
-	  sum += (heat0 - heat1) * ((log(EARTH->bf[t-1]) + log(EARTH->bf[t]))/(2. * EARTH->options->lsteps*EARTH->loci*maxreplicate) + EARTH->bfscale);
-	  //	  printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f) scale=%f\n",myID,t,sum,heat0,heat1,log(EARTH->bf[t-1]),log(EARTH->bf[t]), EARTH->bfscale);
+	      lsum += (heat0 - heat1) * ((EARTH->bf[locus * hc + t-1] + EARTH->bf[locus * hc + t])/(2. * EARTH->options->lsteps*maxreplicate));
+	      //	  printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f) scale=%f\n",myID,t,sum,heat0,heat1,log(EARTH->bf[t-1]),log(EARTH->bf[t]), EARTH->bfscale);
+	    }
+	  bfsum += log(lsum) + EARTH->bfscale[locus];
 	}
-      fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood)\n");
-      fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
-      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f %s\n", sum, 
-	      options->adaptiveheat ? "(adaptive heating -> value may be wrong)" : "");
-      fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n", EARTH->bfscale - log (EARTH->hm / (EARTH->options->lsteps*EARTH->loci*maxreplicate)));
-      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", EARTH->bfscale + log (EARTH->am / (EARTH->options->lsteps*EARTH->loci*maxreplicate)));
-      fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n(3) maybe strongly biased by large values\n\n\n");
-#ifdef PRETTY
-      if(file==EARTH->outfile)
-	pdf_bayes_factor(EARTH,sum, maxreplicate);
-#endif
+      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f %s\n", bfsum, 
+      	      options->adaptiveheat ? "(adaptive heating -> value may be wrong)" : "");
     }
   else
     {
-      fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood)\n");
-      fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
       fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))= UNAVAILABLE because no heating was used\n");
-      fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n", EARTH->bfscale - log (EARTH->hm / (EARTH->options->lsteps*EARTH->loci*maxreplicate)));
-      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", EARTH->bfscale + log (EARTH->am / (EARTH->options->lsteps*EARTH->loci*maxreplicate)));
-      fprintf(file,"(2) is considered a crude estimate; (3) maybe strongly biased by large values\n\n\n");
+    }
+  //  printf("%i> %li %f %f",myID, 0L, EARTH->hmscale[0],EARTH->amscale[0]);
+  //printf(" %f %f\n",EARTH->hm[0],EARTH->am[0]);
+  hsum =  0.0;
+  for(locus=0;locus < EARTH->loci; locus++)
+    {
+      //  printf("%i> %li %f %f",myID, locus, EARTH->hmscale[locus],EARTH->amscale[locus]);
+      //printf(" %f %f\n",EARTH->hm[locus],EARTH->am[locus]);
+      if(EARTH->data->skiploci[locus])
+	continue;
+      hsum += EARTH->hmscale[locus] - log (EARTH->hm[locus] / (EARTH->options->lsteps*maxreplicate));
+    }
+  fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n",hsum);
+      asum =  0.0;
+      //      for(locus=0;locus < EARTH->loci; locus++)
+      //	{
+      //	  if(EARTH->data->skiploci[locus])
+      //	    continue;
+      //	  asum += EARTH->amscale[locus]  + log (EARTH->am[locus] / (EARTH->options->lsteps*maxreplicate));
+      //	}
+      //      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", asum);
+      fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
+      //(3) maybe strongly biased by large values\n\n\n");
 #ifdef PRETTY
       if(file==EARTH->outfile)
-	pdf_bayes_factor(EARTH,sum, maxreplicate);
+	pdf_bayes_factor(EARTH, bfsum, hsum, asum, maxreplicate);
 #endif
-    }
-}
-  
+} 
 
 /// calculate bayes factor using thermodynamic integration
 /// based on a method by Friel and Pettitt 2005
@@ -2207,34 +2412,48 @@ void calculate_BF(world_fmt **universe, option_fmt *options)
 {
   long i;
   MYREAL xx;
-  //  printf("@%i> \t ",myID);
+  //  MYREAL logprior;
+  long locus = universe[0]->locus;
+  long hc = options->heated_chains;
+  if(universe[0]->data->skiploci[locus])
+    return;
+  if(EARTH->likelihood[EARTH->G] <= -HUGE)
+    return;
   if(options->heating)
     {
-      for (i = 0; i < options->heated_chains; i++)
+      for (i = 0; i < hc; i++)
 	{
-	  if(universe[i]->param_like < universe[i]->bfscale)
+	  locus = universe[i]->locus;
+	  xx = universe[i]->likelihood[universe[i]->G];
+#ifdef THERMOCHECK
+	  fprintf(stderr,"@@@ %10.10f %10.10f\n",universe[i]->heat, xx);
+#endif
+	  if(xx < EARTH->bfscale[locus])
 	    {
-	      EARTH->bf[i] += exp(universe[i]->param_like - universe[i]->bfscale);
+	      EARTH->bf[locus * hc + i] += EXP(xx - EARTH->bfscale[locus]);
 	    }
 	  else
 	    {
-	      EARTH->bf[i] *= exp(universe[i]->bfscale - universe[i]->param_like);
-	      universe[i]->bfscale = universe[i]->param_like;
-	      EARTH->bf[i] += exp(universe[i]->param_like - universe[i]->bfscale);
+	      EARTH->bf[locus * hc + i] *= EXP(EARTH->bfscale[locus] - xx);
+	      EARTH->bfscale[locus] = xx;
+	      EARTH->bf[locus * hc + i] += 1.0; //exp(universe[i]->param_like - universe[i]->bfscale);
 	    }
 	}
     }
+  xx = EARTH->likelihood[EARTH->G];
+  if(xx <= -HUGE)
+    return;
+  locus = EARTH->locus;
+  if(xx > EARTH->hmscale[locus])
+    {
+        EARTH->hm[locus] += EXP(EARTH->hmscale[locus] - xx);
+    }
   else
     {
-      EARTH->bfscale = EARTH->param_like;
+      EARTH->hm[locus] *= EXP(xx - EARTH->hmscale[locus]);
+      EARTH->hmscale[locus] = xx;
+      EARTH->hm[locus] += 1.;
     }
-  xx = EXP(EARTH->param_like - EARTH->bfscale);
-  if(xx>SMALLEPSILON)
-    {
-      EARTH->hm += 1. / xx;
-      EARTH->am += xx;
-    }
-  //    printf("%i>\tbf[0]=%f hm=%f am=%f scale=%f plike=%f oldval=%f\n",myID, EARTH->bf[0],EARTH->hm, EARTH->am, EARTH->bfscale, EARTH->param_like,EARTH->bayes->oldval);
 }
 
 
