@@ -16,7 +16,7 @@
  
   Copyright 2002 Peter Beerli and Joseph Felsenstein, Seattle WA
   Copyright 2003 Peter Beerli, Tallahassee FL
-  $Id: migration.h 756 2007-06-27 02:45:26Z beerli $
+  $Id: migration.h 1052 2007-12-03 03:44:31Z beerli $
   *----------------------------------------------------------------
 */
 /**
@@ -133,7 +133,8 @@ typedef struct seqmodel_fmt
     MYREAL freqr, freqy;                    //!< frequency of purins
     MYREAL freqar, freqcy, freqgr, freqty;  //!< frequency of transitions
     MYREAL aa, bb;                          //
-    long endsite;                           //!< number of site patterns
+    long oldsite;                           //!< number of site patterns (raw, with SNP adjustment
+    long endsite;                           //!< number of site patterns 
     MYREAL xi;                              //!< ratio of transitions
     MYREAL xv;                              //!< ratio of transversions
     MYREAL ttratio;                         //!< ration between transitions and transversions
@@ -317,6 +318,7 @@ typedef struct _option
     boolean usertreewithmig;
     boolean randomtree;
     short treeprint;
+  long treeinc;
     boolean usem;
     short migvar;
     boolean plot;
@@ -440,6 +442,8 @@ typedef struct _option
     MYREAL alphavalue;
     MYREAL *mu_rates;
     long muloci;
+  MYREAL *inheritance_scalars;
+  long inheritance_scalars_numalloc;
   boolean has_datefile;
 #ifdef LONGSUM
 
@@ -499,6 +503,7 @@ typedef struct _node
     long pop;
     long actualpop;
     long id;
+  boolean visited;
     xarray_fmt x;
 #ifdef UEP
 
@@ -531,6 +536,7 @@ typedef struct vtlist
     long to;
     /*  long pop; */
     long slice;
+  boolean visited;
 }
 vtlist;
 
@@ -712,6 +718,7 @@ typedef struct _worldoption
     MYREAL *mu_rates;
     MYREAL *lmu_rates;
   MYREAL *meanmu;
+  MYREAL *inheritance_scalars;
 #ifdef LONGSUM
 
     boolean fluctuate;
@@ -749,6 +756,7 @@ typedef struct _worldoption
     long lincr;
     MYREAL loglsteps;
     long treeprint;
+  long treeinc;
     long movingsteps;
     MYREAL acceptfreq;
     long rcategs;
@@ -951,6 +959,8 @@ typedef struct _world
     MYREAL heat;
     MYREAL averageheat;
   MYREAL varheat;
+  MYREAL essminimum;
+  MYREAL logprior;
     long treeswapcount;
     /* reporting time */
     time_t starttime;
@@ -1017,14 +1027,21 @@ typedef struct _world
   MYREAL *besttreelike;
   // Bayes factor
   MYREAL *bf;
-  MYREAL bfscale;
-  MYREAL hm;
-  MYREAL am;
+  MYREAL *bfscale;
+  MYREAL *hmscale;
+  MYREAL *amscale;
+  MYREAL *hm;
+  MYREAL *am;
   // autocorrelation and ESS
   MYREAL *autocorrelation;
   MYREAL *effective_sample;
   MYREAL *auto_archive;
   MYREAL *ess_archive;
+
+  node **node_collection;
+  long node_collection_count;
+  long node_collection_allocated;
+
 }
 world_fmt;
 
