@@ -18,7 +18,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: reporter.c 754 2007-06-18 16:16:01Z beerli $
+$Id: reporter.c 1023 2007-11-13 05:05:42Z beerli $
  
 -------------------------------------------------------*/
 /* \file reporter.c 
@@ -171,7 +171,6 @@ convergence_check_bayes (world_fmt *world,  long maxreplicate)
     if (world->chains == 1 && maxreplicate <= 1)
         return;
     // len defines the length of arrays that
-    // have to hold all km, kt, p, and mindex means
     len = world->numpop2 + 1;
     nmeans  = (long *) mycalloc (maxreplicate, sizeof (long));
     gelmanw = (MYREAL *) mycalloc (len, sizeof (MYREAL));
@@ -180,12 +179,15 @@ convergence_check_bayes (world_fmt *world,  long maxreplicate)
     chain_averages = (MYREAL *) mycalloc (len, sizeof (MYREAL));
 
     // can be done independently before this function
-    // evaluate the within_variances for each monitored parameter: wp[i] = 1/(n-1) Sum[p[i]- mean[p[i]]]
+    // evaluate the within_variances for each monitored parameter: 
+    // wp[i] = 1/(n-1) Sum[p[i]- mean[p[i]]]
     // evaluate the total_within_variance w = 1/m Sum[wp[i]]
     // needs to be done here
-    // evaluate between_variance for each monitored parameter: bp[i] =  Sum[p[j]-mean[p[j]]]
+    // evaluate between_variance for each monitored parameter: 
+    // bp[i] =  Sum[p[j]-mean[p[j]]]
     // where m is the number of indpendent chains, m = 2 ... many, shall we evaluate all pairs?
-    // evaluate between_variance for all parameters b = n/(m-1) .... [check program?
+    // evaluate between_variance for all parameters 
+    // b = n/(m-1) .... [check program?
     // calculate expected variance V = (1-1/n) W + 1/n B
     // Gelman-Rubin statistic sqrt(R) = Sqrt[V/W]
 
@@ -321,14 +323,14 @@ calc_allgelmanb (MYREAL *gelmanb, MYREAL *mc, MYREAL *chainmeans, long *nmeans,
 /// collect the autocorrelation values and the ESS values
 void collect_eff_values(world_fmt *world)
 {
-  const long nn = world->numpop2 + world->bayes->mu;
+  const long nn = world->numpop2 + world->bayes->mu * world->loci + 1;// One is for Log(Prob(Data|Model)
   long i;
   long maxrep = (world->options->replicate == TRUE) ? ((world->options->replicatenum > 0) ? 
 						       world->options->replicatenum  : world->options->lchains ) : 1;
 
   for(i=0;i<nn; i++)
     {
-      world->auto_archive[i] += world->autocorrelation[i]/maxrep;
+      world->auto_archive[i] += world->autocorrelation[i]/maxrep/world->loci;
       world->ess_archive[i] += world->effective_sample[i];
     }
 }
@@ -336,7 +338,7 @@ void collect_eff_values(world_fmt *world)
 
 
 ///
-/// prints order of parameters for header files in bayesfile and bayesallfile
+/// prints the effective sample size and autocorrelation
 void print_param_eff_size(FILE *file, world_fmt *world, long numparam, int offset, 
 			  MYREAL *autocorr, MYREAL *effsample)
 {
@@ -350,37 +352,29 @@ void print_param_eff_size(FILE *file, world_fmt *world, long numparam, int offse
   char *custm2 = world->options->custm2;
   bayes_fmt * bayes = world->bayes;
   char *buffer;
+  long start=0;
 
   if(!world->options->bayes_infer)
-    return;
+    start=world->numpop2;
 
   buffer = (char*) mycalloc(numparam * LINESIZE, sizeof(char));
-  pos = sprintf(buffer,"%*.*sParameter         Autocorrelation      Effective Sample size\n", offset, offset, " ");
+  pos = sprintf(buffer,"%*.*sParameter         Autocorrelation%s   Effective Sample size\n", offset, offset, " ",world->loci>1 ? "(*)" : "   ");
   pos += sprintf(buffer+pos,"%*.*s---------         ---------------      ---------------------\n", offset, offset, " ");
 
-  for(pa0=0;pa0<numparam;pa0++)
+  for(pa0=start;pa0<numparam;pa0++)
     {
-      if(bayes->map[pa0][1] != INVALID)
+      if(pa0 < numpop2)
 	{
-	  if(pa0 == bayes->map[pa0][1])
-	    pa = pa0;
-	  else
-	    pa = bayes->map[pa0][1];
-	  
-	  if(pa0 < numpop)
+	  if(bayes->map[pa0][1] != INVALID)
 	    {
-	      if((pa0 == pa) && (custm2[pa0] == '*'))
-		pos += sprintf(buffer+pos,"%*.*s%s_%li               % -5.5f           %10.2f\n", offset, offset, " ", "Theta",pa0+1,autocorr[pa],effsample[pa]);  
-	      else
-		pos += sprintf(buffer+pos,"%*.*s%s_%li=%li [%c]      % -5.5f        %10.2f\n", offset, offset, " ", 
-		        "Theta",pa0+1, pa+1, custm2[pa0],autocorr[pa],effsample[pa]);  
-	    }
-	  else
-	    {
-	      // do we estimate mutation rate changes?
-	      if(pa0 == numpop2)
+	      pa = bayes->map[pa0][1];
+	      if(pa0 < numpop)
 		{
-		  pos += sprintf(buffer+pos,"%*.*s%s           % -5.5f            %10.2f\n", offset, offset, " ", "Rate",autocorr[pa],effsample[pa]);
+		  if((pa0 == pa) && (custm2[pa0] == '*'))
+		    pos += sprintf(buffer+pos,"%*.*s%s_%li               % -5.5f           %10.2f\n", offset, offset, " ", "Theta",pa0+1,autocorr[pa],effsample[pa]);  
+		  else
+		    pos += sprintf(buffer+pos,"%*.*s%s_%li=%li [%c]      % -5.5f        %10.2f\n", offset, offset, " ", 
+				   "Theta",pa0+1, pa+1, custm2[pa0],autocorr[pa],effsample[pa]);  
 		}
 	      else
 		{
@@ -397,8 +391,28 @@ void print_param_eff_size(FILE *file, world_fmt *world, long numparam, int offse
 		}
 	    }
 	}
+      else		  // do we estimate mutation rate changes?
+	{
+	  if(pa0+1 == numparam)
+	    {
+	      pos += sprintf(buffer+pos,"%*.*s%s         % -5.5f           %10.2f\n", offset, offset, " ", "Ln[Prob(D|P)]",autocorr[pa0],effsample[pa0]);
+	    }
+	  else
+	    {
+	      pos += sprintf(buffer+pos,"%*.*s%s                  % -5.5f           %10.2f\n", offset, offset, " ", "Rate",autocorr[pa0],effsample[pa0]);
+	    }
+	}
     }
-  FPRINTF(file,"%s\n", buffer);
+  if(world->loci>1 && file!=stdout)
+    {
+      pos += sprintf(buffer+pos,"%*.*s(*) averaged over loci.\n", offset, offset, " ");
+    }
+#ifdef MPI
+  if(file==stdout)
+    FPRINTF(file,"[%3i] %s\n", myID, buffer);
+  else
+#endif
+    FPRINTF(file,"%s\n", buffer);
   myfree(buffer);
 }
 
@@ -433,90 +447,122 @@ void print_param_eff_size(FILE *file, world_fmt *world, long numparam, int offse
 /// 
 MYREAL single_chain_var(world_fmt *world, long T, MYREAL *variance, MYREAL *autoc, MYREAL *effsample)
 {
-  static MYREAL *mean;
-  static MYREAL *S;
-  static MYREAL *r;
-  static MYREAL *xold;
-  static MYREAL *xstart;
+  static double *mean;
+  static double *S;
+  static double *r;
+  static double *xold;
+  static double *xstart;
   static boolean done=FALSE;
   static long n = 0;
   static long nn = 0;
   long i;
   long j;
+  long start;
 
-  MYREAL x  = 0.0;
-  MYREAL xo = 0.0;
-  MYREAL x1 = 0.0;
-  MYREAL m  = 0.0;
-  MYREAL mo = 0.0;
-  MYREAL v  = 0.0;
-  MYREAL *delta;
+  double x  = 0.0;
+  double xo = 0.0;
+  double x1 = 0.0;
+  double m  = 0.0;
+  double mo = 0.0;
+  double v  = 0.0;
+  double *delta;
+  double temp;
   if(!done)
     {
-      nn = world->numpop2 + world->bayes->mu;
-      mean = mycalloc(nn,sizeof(MYREAL));
-      S    = mycalloc(nn,sizeof(MYREAL));
-      r    = mycalloc(nn,sizeof(MYREAL));
-      xold = mycalloc(nn,sizeof(MYREAL));
-      xstart = mycalloc(nn,sizeof(MYREAL));
+      nn = world->numpop2 + world->bayes->mu * world->loci + 1;
+      mean = mycalloc(nn,sizeof(double));
+      S    = mycalloc(nn,sizeof(double));
+      r    = mycalloc(nn,sizeof(double));
+      xold = mycalloc(nn,sizeof(double));
+      xstart = mycalloc(nn,sizeof(double));
       done=TRUE;
     }
   // reset static variable for each chain
   if(world==NULL)
     {
       n=0;
-      memset(mean,0,nn * sizeof(MYREAL));
-      memset(S,0,nn * sizeof(MYREAL));
-      memset(r,0,nn * sizeof(MYREAL));
-      memset(xold,0,nn * sizeof(MYREAL));
-      memset(xstart,0,nn * sizeof(MYREAL));
+      memset(mean,0,nn * sizeof(double));
+      memset(S,0,nn * sizeof(double));
+      memset(r,0,nn * sizeof(double));
+      memset(xold,0,nn * sizeof(double));
+      memset(xstart,0,nn * sizeof(double));
       return 0;
     }
-  // hack: change syntax so that an init really works
+  // syntax so that an init really works
   if(T==0)
     return 0;
 
-  delta = mycalloc(nn,sizeof(MYREAL));
-
-  if(n==0)
+  delta = mycalloc(nn,sizeof(double));
+  //handles BAYES or ML
+ start = (world->options->bayes_infer ? 0 : world->numpop2);
+  if(n==0) //initialization of mean calculator
     {
       n += 1;
-      for(i=0; i < nn; i++)
+      for(i=start; i < nn; i++)
 	{
-	  if(world->bayes->map[i][1] == INVALID)
-	    continue;
+	  if(i<world->numpop2)
+	    {
+	      if(world->bayes->map[i][1] == INVALID)
+		continue;
+	      else
+		{
+		  j  = world->bayes->map[i][1];
+		  x         = world->param0[j];
+		}
+	    }
 	  else
 	    {
-	      j  = world->bayes->map[i][1];
+	      j = i;
+	      if(j+1 == nn)
+		{
+		  x = world->likelihood[world->numlike-1];
+		} 
+	      else
+		{
+		  x = world->options->mu_rates[world->locus];
+		}
 	    }
-	  x         = world->param0[j];
-	  delta[j]  = x - mean[j];
-	  mean[j]  += delta[j]/n;
-	  S[j]     += delta[j] * (x - mean[j]);
-	  v        += S[j];
-	}
-      memcpy(xstart,world->param0,sizeof(MYREAL) * world->numpop2);
-      memcpy(xold,world->param0,sizeof(MYREAL) * world->numpop2);
-      if(world->bayes->mu)
-	{
-	  xstart[world->numpop2] = world->options->mu_rates[world->locus];
-	  xold[world->numpop2] = world->options->mu_rates[world->locus];
+	  //	  delta[j]  = x - mean[j];
+	  //mean[j]  += delta[j]/n;
+	  //S[j]     += delta[j] * (x - mean[j]);
+	  //v        += S[j];
+	  mean[j]   = x;
+	  S[j]      = 0.0;
+	  r[j]      = 0.0;
+	  xstart[j] = x;
+	  xold[j] = x;
 	}
+      v = 0.0;
       myfree(delta);
     }
   else
     {
       // n is at least 1
       n += 1;
-      for(i=0; i < nn; i++)
+      for(i=start; i < nn; i++)
 	{
-	  if(world->bayes->map[i][1] == INVALID)
-	    continue;
+	  if(i<world->numpop2)
+	    {
+	      if(world->bayes->map[i][1] == INVALID)
+		continue;
+	      else
+		{
+		  j  = world->bayes->map[i][1];
+		  x  = (double) world->param0[j];
+		}
+	    }
 	  else
 	    {
-	      j  = world->bayes->map[i][1];
+	      j = i;
+	      if(j+1 == nn)
+		{
+		  x = (double) world->likelihood[world->numlike-1];
+		} 
+	      else
+		{
+		  x = (double) world->options->mu_rates[world->locus];
+		}
 	    }
-	  x         = world->param0[j];
 	  xo        = xold[j];
 	  x1        = xstart[j];
 	  delta[j]  = x - mean[j];
@@ -524,23 +570,35 @@ MYREAL single_chain_var(world_fmt *world, long T, MYREAL *variance, MYREAL *auto
 	  mean[j]  += delta[j]/n;
 	  m         = mean[j];
 	  S[j]     += delta[j] * (x - m);
-	  r[j]     += xo * x + mo * (n * mo - x1 - xo) - m * ((n+1) * m - x1 - x);
-	  autoc[j]  = S[j] > 0.0 ? r[j]/S[j]: 1.0;
-	  effsample[j] = n * (1. - autoc[j])/(1. + autoc[j]);
+	  temp = xo * x + mo * (n * mo - x1 - xo) - m * ((n+1) * m - x1 - x);
+	  r[j]     += temp;
+	  autoc[j]  = S[j] > 0.0 ? (MYREAL) (r[j]/S[j]): (MYREAL) 1.0;
+	  effsample[j] = (MYREAL) (n * (1. - autoc[j])/(1. + autoc[j]));
 	  v        += S[j];
-	}
-      //      printf("n=%li effsample=%g atoc=%g r=%g s=%g mean=%g theta1=%g\n", n,effsample[0],autoc[0],r[0],S[0],mean[0],world->param0[0]);
-      memcpy(xold,world->param0,sizeof(MYREAL) * world->numpop2);
-      if(world->bayes->mu)
-	{
-	  xold[world->numpop2] = world->options->mu_rates[world->locus];
+	  //	  printf("[%li] n=%li effsample=%g atoc=%g r=%g (%f) s=%g mean=%g\n", j, n,effsample[j],autoc[j],r[j], temp, S[j],mean[j]);
+	  xold[j] = x;
 	}
       myfree(delta);
     }
-  *variance =  v / n;
+  *variance =  (MYREAL) v / n;
   return *variance;
 }
 
+boolean max_ess(const MYREAL * ess, const long n, const MYREAL minimum)
+{
+  long i;
+  long z=0;
+  for(i=0; i < n; i++)
+    {
+      z += (long) (ess[i] >= minimum);
+    }
+  if(z==n)
+    {
+      return TRUE;
+    }
+  return FALSE;
+}
+
 /// calculate effective sample size
 /// which is the N' = N (1-r)/(1+r)
 /// where r is the autocorrelation coefficient for lag 1.
@@ -807,7 +865,7 @@ MYREAL
 calc_s_bayes (long tthis, MYREAL *tc, world_fmt * world)
 {
   long T            = world->convergence->chain_counts[world->rep];
-  long nn           = 2+world->numpop2 + (world->bayes->mu ? 1 : 0) ;
+  long nn           = 2+world->numpop2 + (world->bayes->mu * world->loci) ;
   //  long pnum         = world->bayes->numparams;
   
   MYREAL  * params     = world->bayes->params;
@@ -883,7 +941,7 @@ chain_means_bayes (MYREAL *thischainmeans, world_fmt * world)
   long              i;
   long              j;
   MYREAL           *params  = world->bayes->params;
-  long              nn      = 2+world->numpop2 + (world->bayes->mu ? 1 : 0) ;
+  long              nn      = 2+world->numpop2 + (world->bayes->mu * world->loci) ;
 
   for (j = 0; j < T; j++)
     {
@@ -903,9 +961,12 @@ chain_means_bayes (MYREAL *thischainmeans, world_fmt * world)
 void
 chain_means (MYREAL *thischainmeans, world_fmt * world)
 {
+  MYREAL temp;
+
   if(world->options->bayes_infer)
     {
-      world->convergence->chain_counts[world->rep] = world->bayes->paramnum;
+      temp = (world->convergence->chain_counts[world->rep]!=0) ? world->convergence->chain_counts[world->rep] : 1;  
+      world->convergence->chain_counts[world->rep] = temp;
       chain_means_bayes (thischainmeans, world);
     }
   else
