@@ -16,7 +16,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: sequence.c 701 2007-05-08 01:57:47Z beerli $
+ $Id: sequence.c 1087 2007-12-20 19:28:34Z beerli $
  
 -------------------------------------------------------*/
 /* \file sequence.c
@@ -73,8 +73,7 @@ void printweights (FILE * outfile, world_fmt * world, option_fmt * options,
                    short inc, long chars, short *weight, char *letters);
 void print_seqfreqs (FILE * outfile, world_fmt * world, option_fmt * options);
 
-void snp_invariants (contribarr invariants, long endsite, long rcategs,
-                     seqmodel_fmt * seq, phenotype x1);
+void snp_invariants (contribarr invariants, world_fmt *world, long locus, phenotype x1);
 
 void makevalues_snp (world_fmt * world, option_fmt * options, data_fmt * data,
                      long locus);
@@ -333,106 +332,109 @@ void constrain_rates(long categs, MYREAL *rate, MYREAL *probcat)
 }
 
 /*data read material ===================================== */
-
+///
+/// read sequence data and linked SNP data 
 void
 make_sequences (world_fmt * world, option_fmt * options, data_fmt * data,
                 long locus)
 {
-    makevalues_seq (world, options, data, locus);
-    if (options->freqsfrom)
+  makevalues_seq (world, options, data, locus);
+  if (options->freqsfrom)
     {
-        empiricalfreqs (world, options, world->data->seq, locus);
-        getbasefreqs (options, world->data->seq, locus);
+      empiricalfreqs (world, options, world->data->seq, locus);
+      getbasefreqs (options, world->data->seq, locus);
     }
 }
 
+/// 
+/// read unlinked SNP data (each locus must be only 1 site)
 void
 make_snp (world_fmt * world, option_fmt * options, data_fmt * data,
           long locus)
 {
-    makevalues_snp (world, options, data, locus);
-    /*if (options->freqsfrom)
-       {
-       empiricalfreqs (world, world->data->seq, locus); */
-    options->freqsfrom = FALSE;
-    getbasefreqs (options, world->data->seq, locus);
-    /* }
-     */
+  makevalues_snp (world, options, data, locus);
+  /*if (options->freqsfrom)
+    {
+    empiricalfreqs (world, world->data->seq, locus); */
+  options->freqsfrom = FALSE;
+  getbasefreqs (options, world->data->seq, locus);
+  /* }
+   */
 }
 
 /* private functions================================== */
 void
 getbasefreqs (option_fmt * options, seqmodel_fmt * seq, long locus)
 {
-    long l;
-    MYREAL aa, bb;
-    if (locus == 0)
-        seq->ttratio = options->ttratio[0];
-    else
+  long l;
+  MYREAL aa, bb;
+  if (locus == 0)
+    seq->ttratio = options->ttratio[0];
+  else
     {
-        for (l = 1; l <= locus; l++)
+      for (l = 1; l <= locus; l++)
         {
-            if (options->ttratio[l] == 0.0)
+	  if (options->ttratio[l] == 0.0)
             {
-                seq->ttratio = options->ttratio[l - 1];
-                break;
+	      seq->ttratio = options->ttratio[l - 1];
+	      break;
             }
-            seq->ttratio = options->ttratio[l];
+	  seq->ttratio = options->ttratio[l];
         }
-        if (l > locus)
-            seq->ttratio = options->ttratio[locus];
+      if (l > locus)
+	seq->ttratio = options->ttratio[locus];
     }
-    check_basefreq (options);
-
+  check_basefreq (options);
+  
 #ifdef ALTIVEC
-    seq->freq.f[0] = seq->freqa = options->freqa;
-    seq->freq.f[1] = seq->freqc = options->freqc;
-    seq->freq.f[2] = seq->freqg = options->freqg;
-    seq->freq.f[3] = seq->freqt = options->freqt;
+  seq->freq.f[0] = seq->freqa = options->freqa;
+  seq->freq.f[1] = seq->freqc = options->freqc;
+  seq->freq.f[2] = seq->freqg = options->freqg;
+  seq->freq.f[3] = seq->freqt = options->freqt;
 #else
-    seq->freqa = options->freqa;
-    seq->freqc = options->freqc;
-    seq->freqg = options->freqg;
-    seq->freqt = options->freqt;    
+  seq->freqa = options->freqa;
+  seq->freqc = options->freqc;
+  seq->freqg = options->freqg;
+  seq->freqt = options->freqt;    
 #endif
-    seq->freqr = seq->freqa + seq->freqg;
-    seq->freqy = seq->freqc + seq->freqt;
-    seq->freqar = seq->freqa / seq->freqr;
-    seq->freqcy = seq->freqc / seq->freqy;
-    seq->freqgr = seq->freqg / seq->freqr;
-    seq->freqty = seq->freqt / seq->freqy;
-    aa =
-        seq->ttratio * (seq->freqr) * (seq->freqy) - seq->freqa * seq->freqg -
-        seq->freqc * seq->freqt;
-    bb = seq->freqa * (seq->freqgr) + seq->freqc * (seq->freqty);
-    seq->xi = aa / (aa + bb);
-    seq->xv = 1.0 - seq->xi;
-    if (seq->xi <= 0.0)
+  seq->freqr = seq->freqa + seq->freqg;
+  seq->freqy = seq->freqc + seq->freqt;
+  seq->freqar = seq->freqa / seq->freqr;
+  seq->freqcy = seq->freqc / seq->freqy;
+  seq->freqgr = seq->freqg / seq->freqr;
+  seq->freqty = seq->freqt / seq->freqy;
+  aa =
+    seq->ttratio * (seq->freqr) * (seq->freqy) - seq->freqa * seq->freqg -
+    seq->freqc * seq->freqt;
+  bb = seq->freqa * (seq->freqgr) + seq->freqc * (seq->freqty);
+  seq->xi = aa / (aa + bb);
+  seq->xv = 1.0 - seq->xi;
+  if (seq->xi <= 0.0)
     {
-        warning ("This transition/transversion ratio (%f)\n",seq->ttratio);
-        warning ("is impossible with these base frequencies (%f, %f, %f, %f)!\n",seq->freqa,seq->freqc,seq->freqg,seq->freqt);
-        seq->xi = 0.00001; // do not set this to zero because of the 1/(fracchange=xi*(...))
-        seq->xv = 0.99999;
-	seq->ttratio =
-	  (seq->freqa * seq->freqg +
-	   seq->freqc * seq->freqt) / ((seq->freqr) * (seq->freqy));
-
-        warning (" Transition/transversion parameter reset\n");
-        warning ("  so transition/transversion ratio is %10.6f\n\n",
-                (seq->ttratio));
+      warning ("This transition/transversion ratio (%f)\n",seq->ttratio);
+      warning ("is impossible with these base frequencies (%f, %f, %f, %f)!\n",seq->freqa,seq->freqc,seq->freqg,seq->freqt);
+      seq->xi = 0.00001; // do not set this to zero because of the 1/(fracchange=xi*(...))
+      seq->xv = 0.99999;
+      seq->ttratio =
+	(seq->freqa * seq->freqg +
+	 seq->freqc * seq->freqt) / ((seq->freqr) * (seq->freqy));
+      
+      warning (" Transition/transversion parameter reset\n");
+      warning ("  so transition/transversion ratio is %10.6f\n\n",
+	       (seq->ttratio));
     }
-    // use 1/frac as precomputation speed up
-    seq->fracchange = 1. / (
-        (seq->xi) * (2 * seq->freqa * (seq->freqgr) +
-                     2 * seq->freqc * (seq->freqty)) + (seq->xv) * (1.0 -
-                             seq->freqa *
-                             seq->freqa -
-                             seq->freqc *
-                             seq->freqc -
-                             seq->freqg *
-                             seq->freqg -
-                             seq->freqt *
-                             seq->freqt));
+  // use 1/frac as precomputation speed up
+  seq->fracchange = 1. / (
+			  (seq->xi) * (2 * seq->freqa * (seq->freqgr) +
+				       2 * seq->freqc * (seq->freqty)) + (seq->xv) * (1.0 -
+										      seq->freqa *
+										      seq->freqa -
+										      seq->freqc *
+										      seq->freqc -
+										      seq->freqg *
+										      seq->freqg -
+										      seq->freqt *
+										      seq->freqt));
 }
 
 /*===================================================*/
@@ -470,13 +472,19 @@ makeweights (world_fmt * world, data_fmt * data, long locus)
 void
 init_sequences2 (world_fmt * world, seqmodel_fmt * seq, long locus)
 {
-    if (locus == 0)
+    if (world->contribution == NULL)
+      {
         world->contribution =
-            (contribarr *) mymalloc (seq->endsite * sizeof (contribarr));
+            (contribarr *) mymalloc ((4 + seq->endsite) * sizeof (contribarr));
+	//	printf("%i> temp=%f size=%li: world->contribution allocated\n",myID, world->heat,(4 + seq->endsite) * sizeof (contribarr));
+      }
     else
+      {
         world->contribution =
-            (contribarr *) myrealloc (world->contribution,
-                                    seq->endsite * sizeof (contribarr));
+	  (contribarr *) myrealloc (world->contribution,
+                                    (4 + seq->endsite) * sizeof (contribarr));
+	//	printf("%i> temp=%f size=%li: world->contribution REallocated\n",myID, world->heat,(4 + seq->endsite) * sizeof (contribarr));
+      }
 }
 
 #ifdef ALTIVEC
@@ -787,10 +795,10 @@ makevalues_snp (world_fmt * world, option_fmt * options, data_fmt * data,
     
     node **treenode = world->nodep;
     f = world->data->seq->addon;
-    for (k = 0; k < world->data->seq->endsite * (world->data->seq->addon + 1);
+    for (k = 0; k < world->data->seq->endsite * (f + 1);
             k += (1 + world->data->seq->addon))
     {
-        j = world->data->seq->alias[k / (world->data->seq->addon + 1)] - 1;
+        j = world->data->seq->alias[k / (f + 1)] - 1;
         i = -1;
         for (pop = 0; pop < world->numpop; pop++)
         {
@@ -1537,7 +1545,7 @@ init_tbl (world_fmt * world, long locus)
         }
     }
     sumrates = 0.0;
-    for (i = 0; i < world->data->seq->endsite - world->data->seq->addon; i++)
+    for (i = 0; i < world->data->seq->oldsite; i++)
     {
         for (j = 0; j < rcategs; j++)
             sumrates +=
@@ -1633,7 +1641,10 @@ printweights (FILE * outfile, world_fmt * world, option_fmt * options,
 void
 print_seqfreqs (FILE * outfile, world_fmt * world, option_fmt * options)
 {
-    if (world->locus == 0 || outfile == stdout)
+  if(outfile==NULL)
+    return;
+  
+  if (world->locus == 0 || outfile == stdout)
     {
         if (options->freqsfrom)
             FPRINTF (outfile, "\nEmpirical ");
@@ -1782,30 +1793,55 @@ treelike_seq (world_fmt * world, long locus)
 }    /* treelikelihood */
 
 void
-snp_invariants (contribarr invariants, long endsite, long rcategs,
+snp_invariants_original (contribarr invariants, long endsite, long rcategs,
                 seqmodel_fmt * seq, phenotype x1)
 {
     long i, j;
-    memset (invariants, 0, sizeof (contribarr));
-    for (i = endsite - seq->addon; i < endsite - seq->addon + 4; i++)
-    {
-        for (j = 0; j < rcategs; j++)
-        {
-#ifdef ALTIVEC
-            invariants[j] +=
-                (seq->freqa * x1[i][j].f[0] + seq->freqc * x1[i][j].f[1] +
-                 seq->freqg * x1[i][j].f[2] + seq->freqt * x1[i][j].f[3]);
-#else
-            invariants[j] +=
-                (seq->freqa * x1[i][j][0] + seq->freqc * x1[i][j][1] +
-                 seq->freqg * x1[i][j][2] + seq->freqt * x1[i][j][3]);
-#endif
-        }
-    }
+    MYREAL *val;
+        memset (invariants, 0, sizeof (contribarr));
     for (j = 0; j < rcategs; j++)
-        invariants[j] = 1. - invariants[j];
+      {
+	for (i = endsite; i < endsite + 4; i++)
+	  {
+	    val = x1[i][j];
+            invariants[j] += 
+	      (seq->freqa * val[0] + seq->freqc * val[1] +
+	       seq->freqg * val[2] + seq->freqt * val[3]);
+	  }
+      }
+    for (j = 0; j < rcategs; j++)
+      invariants[j] = 1. - invariants[j];
 }
 
+void
+snp_invariants (contribarr invariants, world_fmt *world, long locus, phenotype x1)
+{
+    worldoption_fmt *opt;
+    seqmodel_fmt *seq =  world->data->seq;
+    MYREAL summ;
+
+
+    long i, j;
+
+    node *p;
+    opt = world->options;
+    p = crawlback (world->root->next);
+    summ = 0.0;
+    /* snp invariants */
+    for (j = 0; j < opt->rcategs; j++)
+      {
+	invariants[j] = 0.0;
+	for (i = seq->endsite - seq->addon ; i < seq->endsite; i++)
+	  {
+            invariants[j] +=
+	      (seq->freqa * (*x1)[i][0] + seq->freqc * (*x1)[i][1] +
+	       seq->freqg * (*x1)[i][2] + seq->freqt * (*x1)[i][3]);
+	  }
+	invariants[j] = 1 - invariants[j];
+    }
+    //    printf("invariant0=%f\n",invariants[0]);
+}    /* snp_invariants*/
+
 MYREAL
 treelike_snp (world_fmt * world, long locus)
 {
@@ -1817,38 +1853,38 @@ treelike_snp (world_fmt * world, long locus)
     long i, j, k, lai;
 
     node *p;
-#ifdef ALTIVEC
-    vector float *x1;
-    FloatVec freq = seq->freq;
-#else
     sitelike *x1;
-#endif
     opt = world->options;
     p = crawlback (world->root->next);
     summ = 0.0;
     /* snp invariants */
-    snp_invariants (invariants, seq->endsite, opt->rcategs, seq, p->x.s);
+    snp_invariants (invariants,world, locus, p->x.s);
+    for (j = 0; j < opt->rcategs; j++)
+      {
+	if(invariants[j] <= 0.0)
+	  {
+#ifdef DEBUG
+	    printf("invariants are funny %f\n", invariants[j]);
+#endif
+	    return -MYREAL_MAX;
+	  }
+      }
     for (i = 0; i < seq->endsite - seq->addon; i++)
     {
         scale = p->scale[i];
         for (j = 0; j < opt->rcategs; j++)
         {
-#ifdef ALTIVEC
-            x1 = &(p->x.s[i][j].v);
-            tterm[0] = vector_multiply(&freq.v,x1);
-#else
             x1 = &(p->x.s[i][j]);
             tterm[j] =
                 (seq->freqa * (*x1)[0] + seq->freqc * (*x1)[1] +
                  seq->freqg * (*x1)[2] + seq->freqt * (*x1)[3]) /invariants[j];
-#endif
         }
         sumterm = 0.0;
         for (j = 0; j < opt->rcategs; j++)
             sumterm += opt->probcat[j] * tterm[j];
         lterm = LOG (sumterm) + scale;
         for (j = 0; j < opt->rcategs; j++)
-            world->contribution[i][j] = tterm[j] / sumterm;
+	  world->contribution[i][j] = tterm[j] / sumterm;
 
         summ += seq->aliasweight[i] * lterm;
     }    /* over endsite - 4[snp-invariants] */
@@ -1906,9 +1942,9 @@ treelike_snp_unlinked (world_fmt * world, long locus)
     p = crawlback (world->root->next);
     summ = 0.0;
     /* snp invariants */
-    snp_invariants (invariants, seq->endsite, opt->rcategs, seq, p->x.s);
+    snp_invariants (invariants, world, locus, p->x.s);
     /* no rate categories used */
-    for (i = 0; i < seq->endsite - seq->addon; i++)
+    for (i = 0; i < seq->endsite; i++)
     {
         ii = i / 5;
         scale = p->scale[i];
@@ -2059,10 +2095,15 @@ void find_rates_fromdata(data_fmt * data, option_fmt * options)
   n = 0;
   mean = 0.;
   if(options->mu_rates == NULL)
-    options->mu_rates = (MYREAL *) mycalloc( loci, sizeof(MYREAL));
+    {
+      options->mu_rates = (MYREAL *) mycalloc( loci, sizeof(MYREAL));
+      //      printf("%i> opt murate size %li\n",myID,loci * sizeof (MYREAL));
+    }
   n = 0;
   mean = 0. ;
   options->muloci = loci;
+  fprintf(stdout,"Relative mutation rate among loci estimated from the data\n");
+  fprintf(stdout,"---------------------------------------------------------\n");
   for (locus=0; locus < loci; locus++)
     {
       segreg = 0;
@@ -2090,8 +2131,9 @@ void find_rates_fromdata(data_fmt * data, option_fmt * options)
     {
       options->murates = FALSE;
       options->murates_fromdata = FALSE;
-      warning("Relative mutation rates estimation from data was requested but failed, reset to all-equal rates");
+      warning("Relative mutation rates estimation from data was requested but attempt failed --> reset to all-equal rates");
     }
+  fprintf(stdout,"\n\n\n");
   myfree(v[0]);
   myfree(v);
   myfree(numind);
