@@ -20,9 +20,13 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: tools.c 758 2007-07-03 14:12:40Z beerli $
+$Id: tools.c 1090 2007-12-25 20:02:46Z beerli $
 -------------------------------------------------------*/
 /* \file tools.c */
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "migration.h"
 #include "sighandler.h"
 #include "data.h"
@@ -41,7 +45,9 @@ $Id: tools.c 758 2007-07-03 14:12:40Z beerli $
 
 /* external globals*/
 extern int myID;
-
+#ifdef CAUTIOUS
+extern boolean cautious;
+#endif
 /* prototypes ------------------------------------------- */
 MYREAL lengthof (node * p);
 MYINLINE node *crawlback (const node * theNode);
@@ -80,9 +86,11 @@ void onepass_mean_std_end(MYREAL *mean, MYREAL *std, long *n);
 /* vector initialization */
 void doublevec1d (MYREAL **v, long size);
 void doublevec2d (MYREAL ***v, long size1, long size2);
+void floatvec2d (float ***v, long size1, long size2);
 void charvec1d (char **v, long size1);
 void charvec2d (char ***v, long size1, long size2);
 void free_doublevec2d (MYREAL **v);
+void free_floatvec2d (float **v);
 void free_charvec2d (char **v);
 void setdoublevec1d (MYREAL **v, MYREAL *w, long size);
 void add_vector (MYREAL *result, MYREAL *v, long size);
@@ -136,6 +144,13 @@ MYREAL chisquare (long df, MYREAL alpha);
 
 MYREAL chiboundary (long zeros, long nonzeros, MYREAL alpha);
 
+
+#ifdef MESS
+long *check_collection;
+long check_collection_count;
+extern long unique_id_global;
+#endif
+
 /* vector initialization */
 void
 doublevec1d (MYREAL **v, long size)
@@ -143,6 +158,7 @@ doublevec1d (MYREAL **v, long size)
     *v = (MYREAL *) mycalloc (size, sizeof (MYREAL));
 }
 
+/// allocate a 2D array with MYREALS
 void
 doublevec2d (MYREAL ***v, long size1, long size2)
 {
@@ -155,6 +171,19 @@ doublevec2d (MYREAL ***v, long size1, long size2)
     }
 }
 
+/// allocate a 2D array with floats
+void
+floatvec2d (float ***v, long size1, long size2)
+{
+    long i;
+    *v = (float **) mycalloc (size1, sizeof (float *));
+    (*v)[0] = (float *) mycalloc (size1 * size2, sizeof (float));
+    for (i = 1; i < size1; i++)
+    {
+        (*v)[i] = (*v)[0] + size2 * i;
+    }
+}
+
 ///
 /// allocates columns and rows for a matrix of strings
 /// use freecharvec2d() to free this.
@@ -170,12 +199,21 @@ charvec2d (char ***v, long size1, long size2)
     }
 }
 
+/// free 2D vector of MYREAL
 void
 free_doublevec2d (MYREAL **v)
 {
     myfree(v[0]);
     myfree(v);
 }
+
+/// free 2D vector of floats
+void
+free_floatvec2d (float **v)
+{
+    myfree(v[0]);
+    myfree(v);
+}
 void
 free_charvec2d (char **v)
 {
@@ -306,6 +344,245 @@ void onepass_mean_std_end(MYREAL *mean, MYREAL *std, long *n)
   (*std) = sqrt((*std) / ((*n) - 1));
 }
 
+/*garbage collection and recycle*/
+#ifdef MESS
+boolean find_in_list(long id)
+{
+  long i;
+  for(i=0; i < check_collection_count; i++)
+    {
+      if(id == check_collection[i])
+	return TRUE;
+    }
+  return FALSE;
+}
+
+/// tags the lineage that was added the last time the tree was changed.
+///
+boolean traverse_check_id(node *theNode, long id)
+{
+  boolean found = FALSE;
+  if(theNode->type=='r')
+    { 
+      if(theNode->id == id || theNode->next->id == id || theNode->next->next->id == id)
+	return TRUE;	
+      else
+	{
+	  found = traverse_check_id(theNode->next->back, id);
+	  return found;
+	}
+    }
+  if(theNode->type == 'i')
+    {
+      if(theNode->id == id || theNode->next->id == id || theNode->next->next->id == id)
+	return TRUE;	
+      else
+	{
+	  found = traverse_check_id(theNode->next->back, id);
+	  if(found == TRUE)
+	    return found;
+	  found = traverse_check_id(theNode->next->next->back, id);
+	  return found;
+	}
+    }
+  if(theNode->type == 'm')
+    {
+      if(theNode->id == id || theNode->next->id == id)
+	return TRUE;	
+      else
+	{
+	  found = traverse_check_id(theNode->next->back, id);
+	  return found;
+	}
+    }
+  if(theNode->type == 't')
+    {
+      if(theNode->id == id)
+	return TRUE;	
+      else
+	return FALSE;
+    }
+  error("node with no type");
+  return FALSE;
+}
+
+void traverse_checker(node *root)
+{
+  long i;
+  long x;
+  for(i=0;i<check_collection_count; i++)
+    {
+      x = check_collection[i];
+      if(!traverse_check_id(root,x))
+	printf ("Node id %li not found\n",x);
+    }
+}
+#endif /*MESS*/
+
+void start_node_collection(world_fmt *world)
+{
+  long i;
+#ifdef MESS
+  unique_id_global=0;
+#endif
+  world->node_collection_allocated = 2 * world->sumtips  + 2 * world->numpop * MIGRATION_LIMIT;
+  world->node_collection = (node **) calloc(world->node_collection_allocated,sizeof(node*));
+#ifdef MESS
+  check_collection = (long *) calloc(world->node_collection_allocated, sizeof(long));
+#endif
+  for(i = 0; i < world->node_collection_allocated; i++)
+    {
+      world->node_collection[i] = (node *) calloc(1, sizeof(node));
+#ifdef MESS
+      (*world->node_collection[i]).id = unique_id_global++;
+#endif
+    }
+  world->node_collection_count = world->node_collection_allocated;
+}
+
+void stop_node_collection(world_fmt *world)
+{
+  long i;
+  for(i=0;i<world->node_collection_count;i++)
+    {
+      if(world->node_collection[i] != NULL)
+	{
+	  free(world->node_collection[i]);
+	}
+    }
+  myfree(world->node_collection);
+}
+
+void collect_nodelet(world_fmt *world, node *p)
+{
+  //long i;
+  long oldalloc;
+
+  if(world->node_collection_count < world->node_collection_allocated)
+    {
+      world->node_collection[world->node_collection_count] = p;
+#ifdef MESS
+      for(i=0;i<check_collection_count;i++)
+	{
+	  if(p->id == check_collection[i])
+	    break;
+	}
+      if(i==check_collection_count)
+	{
+	  warning("found node not accounted for: %li",p->id);
+	  error("");
+	}
+      else
+	{
+	  check_collection_count--;
+	  check_collection[i] = check_collection[check_collection_count];
+	}
+#endif      
+      world->node_collection_count++;
+      // printf("d%li %c\n",p->id, p->type);
+    }
+  else
+    {
+      warning("in node collection phase we should not need to allocate new space\n");
+      oldalloc = world->node_collection_allocated;
+      world->node_collection_allocated += 400;
+      world->node_collection = (node **) realloc(world->node_collection, world->node_collection_allocated * sizeof(node*));
+      memset(world->node_collection+oldalloc,0, 400 * sizeof(node*));
+      world->node_collection[world->node_collection_count] = p;
+#ifdef MESS
+      for(i=0;i<check_collection_count;i++)
+	{
+	  if(p->id == check_collection[i])
+	    break;
+	}
+      if(i==check_collection_count)
+	{
+	  warning("found node not accounted for: %li",p->id);
+	  error("");
+	}
+      else
+	{
+	  check_collection_count--;
+	  check_collection[i] = check_collection[check_collection_count];
+	}
+#endif
+      world->node_collection_count++;
+      FPRINTF(stdout, "%i> Heat=%f: Collect_nodelet: recycler increased size to: %li/%li\n", myID, 1./world->heat, world->node_collection_count, world->node_collection_allocated);
+    }
+  //printf("\n");
+}
+
+node *  dispense_nodelet(world_fmt *world)
+{
+  static long oldalloc ;
+  static boolean set=FALSE;
+  //static char * treespace;
+  //long allocval;
+  //long pos;
+  
+  long i;
+  node *p = NULL;
+  if(!set)
+    {
+      oldalloc= world->node_collection_allocated;
+      set = TRUE;
+    }
+  if(world->node_collection_count > 0)
+    {
+      world->node_collection_count--;
+      p = world->node_collection[world->node_collection_count];
+#ifdef MESS
+      check_collection[check_collection_count++] = p->id;
+#endif
+      world->node_collection[world->node_collection_count] = NULL;
+    }
+  else
+    {
+      oldalloc += 300;
+      for(i = 0; i < 300; i++)
+	{
+	  world->node_collection[i] = (node *) calloc(1, sizeof(node));
+#ifdef MESS
+	  world->node_collection[i]->id = unique_id_global++;
+#endif
+	}
+#ifdef MESS
+      check_collection = (long *) realloc(check_collection, oldalloc * sizeof(long));
+#endif
+      world->node_collection_count += 300;
+      world->node_collection_count--;
+      p = world->node_collection[world->node_collection_count];
+#ifdef MESS
+      check_collection[check_collection_count++] = p->id;
+#endif
+      world->node_collection[world->node_collection_count] = NULL;
+
+    }
+  return p;
+}
+
+///
+/// swaps the nodelet recycler between different temperatured chains
+/// 
+void swap_node_collection(world_fmt * tthis, world_fmt * that)
+{
+  node **temp;
+  //long tempalloc;
+  long tempcount;
+  long tempallocated;
+  temp = tthis->node_collection;
+  tempcount = tthis->node_collection_count;
+  tempallocated = tthis->node_collection_allocated;
+
+  tthis->node_collection = that->node_collection;
+  tthis->node_collection_count = that->node_collection_count;
+  tthis->node_collection_allocated = that->node_collection_allocated;
+
+  that->node_collection = temp;
+  that->node_collection_count = tempcount;
+  that->node_collection_allocated = tempallocated;
+}
+
 
 /*FILEMANIPULATION======================================================= */
 void
@@ -544,6 +821,16 @@ translate (char *text, char from, char to)
     }
     text[j] = '\0';
 }
+/// unpad from the end of the word until there is none of the
+/// specified characeter left
+void
+unpad (char *text, char removechars[])
+{
+  int gap = strlen(text)-1;
+  while (gap > 0 && strchr(removechars,text[gap]))
+        gap--;
+    text[gap+1] = '\0';
+}
 
 ///
 /// get next word from list with list of delimiters
@@ -684,7 +971,7 @@ void unread_word(FILE *infile, char *word)
 extern int errno;
 #endif
 
-/// opns a file for reading or writing
+/// opens a file for reading or writing
 void
 openfile (FILE ** fp, char *filename, char *mode, char *perm)
 {
@@ -692,11 +979,40 @@ openfile (FILE ** fp, char *filename, char *mode, char *perm)
     FILE *of = NULL;
     char *file;
     char *p;
+#ifdef CAUTIOUS
+    struct stat sb;
+#endif
     file = (char *) mycalloc(LINESIZE,sizeof(char));
     if ((p = strpbrk (filename, CRLF)) != NULL)
         *p = '\0';
     remove_trailing_blanks(&filename);
     strncpy (file, filename, LINESIZE - 1);
+#ifdef CAUTIOUS
+    if(cautious)
+      {
+	//	sb = (stat *) mycalloc(1,sizeof(stat));
+	if(strchr(mode,'w') && (0 == stat(file, &sb)))
+	  {
+	    file[0] = '\0';
+	    while (file[0] == '\0' && trials++ < 10)
+	      {
+		printf ("File %s exists, do you want to overwrite it? [YES/No]\n===>",filename);
+		FGETS (file, LINESIZE, stdin);
+		if(file[0]=='Y' || file[0] == 'y')
+		  {
+		    file[0] = '\0';
+		    while (file[0] == '\0' && trials++ < 10)
+		      {
+			printf ("Please enter a new filename>");
+			FGETS (file, LINESIZE, stdin);
+		      }
+		    
+		  }
+	      }
+	  }
+	//	myfree(sb);
+      }
+#endif
     while (trials++ < 10)
     {
       of = fopen (file, mode);
@@ -1151,8 +1467,10 @@ void
 insert_migr_node (world_fmt * world, node * up, node * down,
                   migr_table_fmt * migr_table, long *migr_table_counter)
 {
-    long i, panic;
-    node *tmp, *tmp2, *oldNode, *oldNode2, *theNode;
+  long i;
+  //, panic;
+    //node *tmp, *tmp2, *oldNode, *oldNode2, *theNode;
+  node *theNode;
     if (!up->top)
         error ("up has to be a top-node");
     theNode = showtop (up)->back;
@@ -1167,14 +1485,26 @@ insert_migr_node (world_fmt * world, node * up, node * down,
     if (((*migr_table_counter) > 0)
             && (migr_table[(*migr_table_counter) - 1].from != down->actualpop))
         error ("problem catched in inser_migr_table");
+    theNode=up;
     for (i = 0; i < (*migr_table_counter); i++)
     {
+      theNode = add_migration(world, theNode,
+			      migr_table[i].from, 
+			      migr_table[i].to,
+			      migr_table[i].time - theNode->tyme);
+    }
+    //printf("%li migration nodelets added\n",(*migr_table_counter) * 2);
+    down->back = theNode;
+    theNode->back = down;
+      /* the stuff below will be excised once I am complete sure that
+	 this recycler above really works -- it seems currently dec 05 07
         tmp = (node *) mycalloc (1, sizeof (node));
         tmp2 = (node *) mycalloc (1, sizeof (node));
         oldNode = up;
         theNode = up->back;
         panic = 0;
-        while (theNode->tyme < migr_table[i].time /*&& panic++ < 10000*/)
+	//printf("+");
+        while (theNode->tyme < migr_table[i].time ****&& panic++ < 10000****)
         {
             if (theNode->type == 't' )
             {
@@ -1207,7 +1537,6 @@ insert_migr_node (world_fmt * world, node * up, node * down,
         tmp->next = tmp2;
         tmp2->next = tmp;
         tmp2->top = 1;
-
         oldNode2 = down;
         theNode = down->back;
         while (theNode->tyme > migr_table[i].time)
@@ -1217,7 +1546,7 @@ insert_migr_node (world_fmt * world, node * up, node * down,
         }
         tmp2->back = oldNode2;
         oldNode2->back = tmp2;
-    }
+}*/
 }
 
 
@@ -1263,7 +1592,7 @@ mylgamma (MYREAL z)
     MYREAL tmp;
     if (z <= 0.)
     {
-        return DBL_MAX;  /*this will kill the receiving calculation */
+        return MYREAL_MAX;  /*this will kill the receiving calculation */
     }
     result = 0.;
     tmp = z + 7.;
@@ -1834,7 +2163,7 @@ d1mach (long i)
   //const  MYREAL numbermin = FLT_MIN;
   //#else
   const  MYREAL eps = DBL_EPSILON ;
-  const  MYREAL numbermax = DBL_MAX;
+  const  MYREAL numbermax = MYREAL_MAX;
   const  MYREAL numbermin = DBL_MIN;
   //#endif
 
@@ -2656,7 +2985,7 @@ void
 gamma_rates (MYREAL *rate, MYREAL *probcat, long categs, char *input)
 {
     long i;
-    MYREAL alpha = DBL_MAX;
+    MYREAL alpha = MYREAL_MAX;
     MYREAL value;
     while (!isdigit (*input) && *input != '\0')
         input++;
@@ -2861,7 +3190,6 @@ long print_to_buffer(char **buffer, long *maxbufsize, long *pos, const char *fmt
 }
 
 
-
 // calculates a rough approximation to log(val)
 // max error = 0.0007
 // Submitted by Laurent de Soras, posted on 30 March 2001
