@@ -32,24 +32,24 @@ double watterson_b(long n)
   return sum;
 }
 
-double watterson(long segreg, long n)
+MYREAL watterson(long segreg, long n)
 {
-  return segreg / watterson_a(n);
+  return (MYREAL) (segreg / watterson_a(n));
 }
 
 //wattersonvar[s_, n_] :=
 //((an = (Sum[1./k, {k, 1,
 // n}]))(thetaw = watterson[s, n]) +  (bn = (Sum[
 // 1./(k^2), {k, 1, n}]))thetaw^2 )/(an^2)
-double wattersonvar(double  thetaw, long n)
+MYREAL wattersonvar(double  thetaw, long n)
 {
   double an = watterson_a(n);
   double bn = watterson_b(n);
-  return (an * thetaw + bn * thetaw * thetaw) / (an*an);
+  return (MYREAL) ((an * thetaw + bn * thetaw * thetaw) / (an*an));
 }
 
-double wattersonstd(double  thetaw, long n)
+MYREAL wattersonstd(double  thetaw, long n)
 {
-  return sqrt(wattersonvar(thetaw,n));
+  return (MYREAL)(sqrt(wattersonvar(thetaw,n)));
 }
 
